% Begin file
close all

kp1 = -11.25;
tauI1 = 46571.70; 
tauD1 = 8.2e-10; 
tauL1 = 1e-5;

kp2 = 15.49;
tauI2 = 3.87;
tauD2 = 2.27e-7;
tauL2 = 1.22e-5;


omega = logspace(-4,4,61);
g11 = G(1,1);
g22 = G(2,2);

% kp1 = -12;
% tauI1 = 4600; 
% tauD1 = 9e-10; 
% tauL1 = 2e-5;
% 
% kp2 = 14;
% tauI2 = 4;
% tauD2 = 3e-7;
% tauL2 = 2e-5;

k1 = kp1*(1 + 1/(tauI1*s) + tauD1*s)*(1/(tauL1*s+1));
k2 = kp2*(1 + 1/(tauI2*s) + tauD2*s)*(1/(tauL2*s+1));

K = [k1 0;0 k2];

% Individual loopshaping
loop11 = loopsens(g11,k1);
loop22 = loopsens(g22,k2);

bandwidthSys = tf([1/sqrt(2)],[1]);
bandwidthLog = 20*log(abs(1/sqrt(2)))*ones(1,length(omega));

for i= 1:length(omega)
    S11f(i) = freqresp(loop11.Si,omega(i));
    S22f(i) = freqresp(loop22.Si,omega(i));
    H11f(i) = freqresp(loop11.Ti,omega(i));
    H22f(i) = freqresp(loop22.Ti,omega(i));
end
logabsS11 = 20*log(abs(S11f));
logabsS22 = 20*log(abs(S22f));

logabsH11 = 20*log(abs(H11f));
logabsH22 = 20*log(abs(H22f));

% figure(100)
% subplot(2,1,1)
% semilogx(omega,logabsH11,omega,logabsS11,omega,bandwidthLog,'k--')
% grid on
% xlim([0.01 10000]);
% subplot(2,1,2)
% semilogx(omega,logabsH22,omega,logabsS22,omega,bandwidthLog,'k--')
% grid on
% xlim([0.01 10000]);



% Sensitivity for the whole system
wholeLoop = loopsens(G,K);

% Checking nominal stability for the complementary sensitivity function
p = pole(wholeLoop.Ti)

for i = length(p)
   realP = real(p(i));
   if realP >=0
        disp('The system is nominal unstable')
        break
   end
end
if realP<0
    disp('The system is nominal stable')
end

WoH = series(wholeLoop.Ti,Wo);
for i= 1:length(omega)
   WoHftemp(i,:,:) = freqresp(WoH,omega(i));
end
WoHft2(:,1:2) = WoHftemp(:,:,1);
WoHft2(:,3:4) = WoHftemp(:,:,2);

for i= 1:length(omega)
    WoHf3(1,1) =  WoHft2(i,1);
    WoHf3(2,1) =  WoHft2(i,2);
    WoHf3(1,2) =  WoHft2(i,3);
    WoHf3(2,2) =  WoHft2(i,4);
    sigma = svd(WoHf3);
    sigmaMax(i) = sigma(1);
end

figure(1)
loglog(omega,sigmaMax,'linewidth',1.5)
grid on
xlim([omega(1) omega(length(omega))]);

ylab=ylabel('Magnitude');
xlab=xlabel('Frequency $[\omega]$');
set(xlab,'interpreter','latex')
set(ylab,'interpreter','latex')
set(ylab,'FontSize',15)
set(xlab,'FontSize',15)
set(gca,'FontSize',15)
print -dpng grx/WoHBao




% End file

% SIMCsimul;
