% Begin file
close all

omega = logspace(-4,4,61);
g11 = G(1,1);
g22 = G(2,2);

% Uncertanty parameters
theta1 = 0;
theta2 = 0;
kun1 = 1;%0.8;
kun2 = 1;%1.2;

% Desired time konstant from the colosed loop system
tauC1 = 5;
tauC2 = 5;
% tauC1 = 0.1;
% tauC2 = 0.1;
% SIMC settings for PI control
KcSIMC1 = ((1/-33.89)*(98.02/(tauC1+theta1)))
tauISIMC1 = min([98.02 4*(tauC1+theta1)]) % timed by 6 so the output is below one at the overshoot
tauDSIMC1 = 0.42
ISIMC1 = KcSIMC1/tauISIMC1
tauL1 = 0.1*tauDSIMC1

KcSIMC2 = ((1/34.84)*(110.5/(tauC2+theta2)))
tauISIMC2 = min([110.5 4*(tauC2+theta2)])
tauDSIMC2 = 0.03
ISIMC2 = KcSIMC2/tauISIMC2
tauL2 = 0.1*tauDSIMC2

k1 = (KcSIMC1 + ISIMC1/s + KcSIMC1*tauDSIMC1*s)*(1/(tauL1*s+1));
k2 = (KcSIMC2 + ISIMC2/s + KcSIMC2*tauDSIMC2*s)*(1/(tauL2*s+1));


% kp1 = -11.25;
% tauI1 = 46571.70; 
% tauD1 = 8.2e-10; 
% tauL1 = 1e-5;
% 
% kp2 = 15.49;
% tauI2 = 3.87;
% tauD2 = 2.27e-7;
% tauL2 = 1.22e-5;
% 
% k1 = kp1*(1 + 1/(tauI1*s) + tauD1*s)*(1/(tauL1*s+1));
% k2 = kp2*(1 + 1/(tauI2*s) + tauD2*s)*(1/(tauL2*s+1));

K = [k1 0;0 k2];

% Individual loopshaping
loop11 = loopsens(g11,k1);
loop22 = loopsens(g22,k2);

bandwidthSys = tf([1/sqrt(2)],[1]);
bandwidthLog = 20*log(abs(1/sqrt(2)))*ones(1,length(omega));

for i= 1:length(omega)
    S11f(i) = freqresp(loop11.Si,omega(i));
    S22f(i) = freqresp(loop22.Si,omega(i));
    H11f(i) = freqresp(loop11.Ti,omega(i));
    H22f(i) = freqresp(loop22.Ti,omega(i));
end
logabsS11 = 20*log(abs(S11f));
logabsS22 = 20*log(abs(S22f));

logabsH11 = 20*log(abs(H11f));
logabsH22 = 20*log(abs(H22f));

% figure(100)
% subplot(2,1,1)
% semilogx(omega,logabsH11,omega,logabsS11,omega,bandwidthLog,'k--')
% grid on
% xlim([0.01 10000]);
% subplot(2,1,2)
% semilogx(omega,logabsH22,omega,logabsS22,omega,bandwidthLog,'k--')
% grid on
% xlim([0.01 10000]);



% Sensitivity for the whole system
wholeLoop = loopsens(G,K);

% Checking nominal stability for the complementary sensitivity function
p = pole(wholeLoop.Ti)

for i = length(p)
   realP = real(p(i));
   if realP >=0
        disp('The system is nominal unstable')
        break
   end
end
if realP<0
    disp('The system is nominal stable')
end

WoH = series(wholeLoop.Ti,Wo);
for i= 1:length(omega)
   WoHftemp(i,:,:) = freqresp(WoH,omega(i));
end
WoHft2(:,1:2) = WoHftemp(:,:,1);
WoHft2(:,3:4) = WoHftemp(:,:,2);

for i= 1:length(omega)
    WoHf3(1,1) =  WoHft2(i,1);
    WoHf3(2,1) =  WoHft2(i,2);
    WoHf3(1,2) =  WoHft2(i,3);
    WoHf3(2,2) =  WoHft2(i,4);
    sigma = svd(WoHf3);
    sigmaMax(i) = sigma(1);
end

figure(1)
dd = loglog(omega,sigmaMax,'linewidth',1.5);
xlab = xlabel('Frequency $[\omega]$');
ylab = ylabel('Magnitude');
grid on
set(xlab,'interpreter','latex')
set(ylab,'interpreter','latex')
set(ylab,'FontSize',15)
set(xlab,'FontSize',15)
set(gca,'FontSize',15)
print -dpng grx/WoHSIMC



% End file

% SIMCsimul;
