% 
% [valdAk,valdCk,valdDk,valgammak,valdX] =
% optStep(X0,Ag,Bg,Cg,Dg,Ao,Bo,Co,Do,Ap,Bp,Cp,Dp,Ak,Bk,Ck,Dk,epsilon)
%
% Inpus: - state space representation of the system G: Ag,Bg,Cg,Dg
%        - state space representation of the uncertainty wo and performance
%          weight wp: Ao,Bo,Co,Do,Ap,Bp,Cp,Dp
%        - state space representation of the initial values of the controller K: Ak,Bk,Ck,Dk
%        - initial value of the positive definite symetric matrix X0
%        - bound of each element of the optimal step dAk,dBk,dCk,dDk and dX0: epsilon
% 
% Building the LMI
% 
% |dA'*X0+A0'*dX+A0'*X0 + X0*dA+dX*A0+X0A0    dX*B0+X0*dB+X0B0    C'*gamma+C0|
% |           dB'*X0+B0'*dX+B0'*X0                   -I           D'*gamma+D0| <1
% |                gamma*C+C0                    gamma*D+D0           -I     |
% 
%                                    gamma>0
%                                    dX+X0>0
% 
% 
% and solve
%                                    min c'x
%                                      s.t.
% 
% |dA'*X0+A0'*dX+A0'*X0 + X0*dA+dX*A0+X0A0    dX*B0+X0*dB+X0B0    C'*gamma+C0|
% |           dB'*X0+B0'*dX+B0'*X0                   -I           D'*gamma+D0| <1
% |                gamma*C+C0                    gamma*D+D0           -I     |
% 
%                                    gamma>0
%                                    dX+X0>0
% 
% 
% 
% 
% 

function [valdAk,valdCk,valdDk,valgammak,valdX] = optStep(X0,Ag,Bg,Cg,Dg,Ao,Bo,Co,Do,Ap,Bp,Cp,Dp,Ak,Bk,Ck,Dk,epsilon)
%% System matrix decleration
Agsize = size(Ag);
Apsize = size(Ap);
Aosize = size(Ao);
WpbGcsize = size(-Bp*Cg);
WobGcsize = size(Bo*Cg);
Wpbsize = size(Bp);
Wpcsize = size(Cp);
Wocsize = size(Co);
WodGcsize = size(Do*Cg);

A = [Ag zeros(Agsize(1),Apsize(2)) zeros(Agsize(1),Aosize(2));
     -Bp*Cg Ap zeros(WpbGcsize(1),Aosize(2));
     Bo*Cg zeros(WobGcsize(1),Apsize(2)) Ao];

B1 = [zeros(Agsize(1),Wpbsize(2));Bp;zeros(Aosize(1),Wpbsize(2))];
B2 = [Bg;-Bp*Dg;Bo*Dg];

C1u = [zeros(WodGcsize(1),WodGcsize(2)) zeros(Wpcsize(1),Wpcsize(2)) zeros(Wocsize(1),Wocsize(2));
       Do*Cg zeros(Wpcsize(1),Wpcsize(2)) Co];

C2 = [-Cg zeros(2,length(A)-length(Cg))];

gammaC1 = [-Dp*Cg Cp zeros(Wocsize(1),Wocsize(2));
           zeros(WodGcsize(1),WodGcsize(2)) zeros(Wpcsize(1),Wpcsize(2)) zeros(Wocsize(1),Wocsize(2))];

gammaD11 = [Dp; zeros(WodGcsize(1),length(Dp))];

Acl = [A+B2*Dk*C2 B2*Ck;
       Bk*C2 Ak];

% Bcl = [B1+B2*Dk;
%        Bk];

C1usize=size(C1u);
Cclu = [C1u zeros(C1usize(1),length(Ak))];
Cclgamma = [gammaC1 zeros(C1usize(1),length(Ak))];
Dclgamma = gammaD11;

% % Bare for test %
% Dcl = Dclgamma;
% Ccl = Cclu+Cclgamma;

% [Dclrow Dclcol] = size(Dclgamma);

%% LMI variable declareation according to Bao (46)

setlmis([]);
[gamma,kn,sgamma] = lmivar(1,[1 0]);

subX0rowcol1Length = length(Ag);
subX0rowcol2Length = length(Ap);
subX0rowcol3Length = length(Ao);
subX0rowcol4Length = length(Ak);
% Bygger dAk
loopno = 2;
for k = 1:loopno 
    [daki(k),kn(k),sdaki(k)] = lmivar(1,[1 0]);
    [dAki,kn,sdAki] = lmivar(3,[0,0;0,-sdaki(k)]);
    if k==1
        [dAk,kn,sdAk] = lmivar(3,[sdAki,zeros(2,2*(loopno-k))]);
    end
    if k~=1 && k~=loopno
        [dAk,kn,sdAk] = lmivar(3,[sdAk;zeros(2,(2*(k-1))),sdAki,zeros(2,2*(loopno-k))]);
    end
    if k==loopno
        [dAk,kn,sdAk] = lmivar(3,[sdAk;zeros(2,2*(loopno-1)),sdAki]);
    end
end
% Bygger dCk
for k = 1:loopno 
    [dck1i(k),kn(k),sdck1i(k)] = lmivar(1,[1 0]);
    [dck2i(k),kn(k),sdck2i(k)] = lmivar(1,[1 0]);
    [dCki,kn,sdCki] = lmivar(3,[sdck1i(k),sdck2i(k)]);
    
    if k==1
        [dCk,kn,sdCk] = lmivar(3,[sdCki,zeros(1,2*(loopno-k))]);
    end
    if k~=1 && k~=loopno
        [dCk,kn,sdCk] = lmivar(3,[sdCk;zeros(1,(2*(k-1))),sdCki(k),zeros(1,2*(loopno-k))]);
    end
    if k==loopno
      [dCk,kn,sdCk] = lmivar(3,[sdCk;zeros(1,2*(loopno-1)),sdCki]);
    end
end
% Bygger dDk
for k = 1:loopno 
    [dDki(k),kn(k),sdDki(k)] = lmivar(1,[1 0]);
    if k==1
        [dDk,kn,sdDk] = lmivar(3,[sdDki(k),zeros(1,loopno-k)]);
    end
    if k~=1 && k~=loopno
    [Dk,kn,sDk] = lmivar(3,[sdDk;zeros(1,k-1),sDki(k),zeros(1,loopno-k)]);
    end
    if k==loopno
      [dDk,kn,sdDk] = lmivar(3,[sdDk;zeros(1,loopno-1),sdDki(k)]);
    end
end
% Bygger dX elementene
for k = 1:length(Acl)
    for i = k:length(Acl)
        [dX(k,i),kn,sdX(k,i)] =  lmivar(1,[1 0]);
    end
end
sdXT = sdX';
% dX11
for k = 1:subX0rowcol1Length
    if k==1
        [dX11,kn,sdX11] = lmivar(3,sdX(k,k:subX0rowcol1Length));   
    else
        [dX11,kn,sdX11] = lmivar(3,[sdX11;sdXT(k,1:k-1),sdX(k,k:subX0rowcol1Length)]);
    end
end
% dX12
for k = 1:subX0rowcol1Length
    if k==1
        [dX12,kn,sdX12] = lmivar(3,sdX(k,subX0rowcol1Length+1:subX0rowcol1Length+subX0rowcol2Length));   
    else
        [dX12,kn,sdX12] = lmivar(3,[sdX12;sdX(k,subX0rowcol1Length+1:subX0rowcol1Length+subX0rowcol2Length)]);
    end
end
% dX13
for k = 1:subX0rowcol1Length
    if k==1
        [dX13,kn,sdX13] = lmivar(3,sdX(k,subX0rowcol1Length+subX0rowcol2Length+1:subX0rowcol1Length+subX0rowcol2Length+subX0rowcol3Length));   
    else
        [dX13,kn,sdX13] = lmivar(3,[sdX13;sdX(k,subX0rowcol1Length+subX0rowcol2Length+1:subX0rowcol1Length+subX0rowcol2Length+subX0rowcol3Length)]);
    end
end
% dX14
for k = 1:subX0rowcol1Length
    if k==1
        [dX14,kn,sdX14] = lmivar(3,sdX(k,subX0rowcol1Length+subX0rowcol2Length+subX0rowcol3Length+1:length(Acl)));   
    else
        [dX14,kn,sdX14] = lmivar(3,[sdX14;sdX(k,subX0rowcol1Length+subX0rowcol2Length+subX0rowcol3Length+1:length(Acl))]);
    end
end
% dX21
[dX21,kn,sdX21] = lmivar(3,sdX12');
% dX22
for k = subX0rowcol1Length+1:subX0rowcol1Length+subX0rowcol2Length 
    if k==subX0rowcol1Length+1
        [dX22,kn,sdX22] = lmivar(3,sdX(k,k:subX0rowcol1Length+subX0rowcol2Length));   
    else
        [dX22,kn,sdX22] = lmivar(3,[sdX22;sdXT(k,subX0rowcol1Length+1:k-1),sdX(k,k:subX0rowcol1Length+subX0rowcol2Length)]);
    end
end
% dX23
for k = subX0rowcol1Length+1:subX0rowcol1Length+subX0rowcol2Length 
    if k==subX0rowcol1Length+1
        [dX23,kn,sdX23] = lmivar(3,sdX(k,subX0rowcol1Length+subX0rowcol2Length+1:subX0rowcol1Length+subX0rowcol2Length+subX0rowcol3Length));   
    else
        [dX23,kn,sdX23] = lmivar(3,[sdX23;sdX(k,subX0rowcol1Length+subX0rowcol2Length+1:subX0rowcol1Length+subX0rowcol2Length+subX0rowcol3Length)]);
    end
end
% dX24
for k = subX0rowcol1Length+1:subX0rowcol1Length+subX0rowcol2Length 
    if k==subX0rowcol1Length+1
        [dX24,kn,sdX24] = lmivar(3,sdX(k,subX0rowcol1Length+subX0rowcol2Length+subX0rowcol3Length+1:length(Acl)));   
    else
        [dX24,kn,sdX24] = lmivar(3,[sdX24;sdX(k,subX0rowcol1Length+subX0rowcol2Length+subX0rowcol3Length+1:length(Acl))]);
    end
end
% dX31
[dX31,kn,sdX31] = lmivar(3,sdX13');
% dX32
[dX32,kn,sdX32] = lmivar(3,sdX23');
% dX33
for k = subX0rowcol1Length+subX0rowcol2Length+1:subX0rowcol1Length+subX0rowcol2Length+subX0rowcol3Length
    if k==subX0rowcol1Length+subX0rowcol2Length+1
        [dX33,kn,sdX33] = lmivar(3,sdX(k,k:subX0rowcol1Length+subX0rowcol2Length+subX0rowcol3Length));   
    else
        [dX33,kn,sdX33] = lmivar(3,[sdX33;sdXT(k,subX0rowcol1Length+subX0rowcol2Length+1:k-1),sdX(k,k:subX0rowcol1Length+subX0rowcol2Length+subX0rowcol3Length)]);
    end
end
% dX34
for k = subX0rowcol1Length+subX0rowcol2Length+1:subX0rowcol1Length+subX0rowcol2Length+subX0rowcol3Length
    if k==subX0rowcol1Length+subX0rowcol2Length+1
        [dX34,kn,sdX34] = lmivar(3,sdX(k,subX0rowcol1Length+subX0rowcol2Length+subX0rowcol3Length+1:length(Acl)));   
    else
        [dX34,kn,sdX34] = lmivar(3,[sdX34;sdX(k,subX0rowcol1Length+subX0rowcol2Length+subX0rowcol3Length+1:length(Acl))]);
    end
end
% dX41
[dX41,kn,sdX41] = lmivar(3,sdX14');
% dX42
[dX42,kn,sdX42] = lmivar(3,sdX24');
% dX43
[dX43,kn,sdX43] = lmivar(3,sdX34');
% dX44
for k = subX0rowcol1Length+subX0rowcol2Length+subX0rowcol3Length+1:length(Acl)
    if k==subX0rowcol1Length+subX0rowcol2Length+subX0rowcol3Length+1
        [dX44,kn,sdX44] = lmivar(3,sdX(k,k:length(Acl)));   
    else
        [dX44,kn,sdX44] = lmivar(3,[sdX44;sdXT(k,subX0rowcol1Length+subX0rowcol2Length+subX0rowcol3Length+1:k-1),sdX(k,k:length(Acl))]);
    end
end
% Bygger total symetrisk dX matrise
[dXtot,kn,sdXtot]= lmivar(3,[sdX11,sdX12,sdX13,sdX14;sdX21,sdX22,sdX23,sdX24;sdX31,sdX32,sdX33,sdX34;sdX41,sdX42,sdX43,sdX44]);

%% Declaring constant submatices of X0
X011 = X0(1:subX0rowcol1Length,1:subX0rowcol1Length);
X012 = X0(1:subX0rowcol1Length,subX0rowcol1Length+1:subX0rowcol1Length+subX0rowcol2Length);
X013 = X0(1:subX0rowcol1Length,subX0rowcol1Length+subX0rowcol2Length+1:subX0rowcol1Length+subX0rowcol2Length+subX0rowcol3Length);
X014 = X0(1:subX0rowcol1Length,subX0rowcol1Length+subX0rowcol2Length+subX0rowcol3Length+1:subX0rowcol1Length+subX0rowcol2Length+subX0rowcol3Length+subX0rowcol4Length);

X022 = X0(subX0rowcol1Length+1:subX0rowcol1Length+subX0rowcol2Length,subX0rowcol1Length+1:subX0rowcol1Length+subX0rowcol2Length);
X023 = X0(subX0rowcol1Length+1:subX0rowcol1Length+subX0rowcol2Length,subX0rowcol1Length+subX0rowcol2Length+1:subX0rowcol1Length+subX0rowcol2Length+subX0rowcol3Length);
X024 = X0(subX0rowcol1Length+1:subX0rowcol1Length+subX0rowcol2Length,subX0rowcol1Length+subX0rowcol2Length+subX0rowcol3Length+1:subX0rowcol1Length+subX0rowcol2Length+subX0rowcol3Length+subX0rowcol4Length);

X033 = X0(subX0rowcol1Length+subX0rowcol2Length+1:subX0rowcol1Length+subX0rowcol2Length+subX0rowcol3Length,subX0rowcol1Length+subX0rowcol2Length+1:subX0rowcol1Length+subX0rowcol2Length+subX0rowcol3Length);
X034 = X0(subX0rowcol1Length+subX0rowcol2Length+1:subX0rowcol1Length+subX0rowcol2Length+subX0rowcol3Length,subX0rowcol1Length+subX0rowcol2Length+subX0rowcol3Length+1:subX0rowcol1Length+subX0rowcol2Length+subX0rowcol3Length+subX0rowcol4Length);

X044 = X0(subX0rowcol1Length+subX0rowcol2Length+subX0rowcol3Length+1:subX0rowcol1Length+subX0rowcol2Length+subX0rowcol3Length+subX0rowcol4Length,subX0rowcol1Length+subX0rowcol2Length+subX0rowcol3Length+1:subX0rowcol1Length+subX0rowcol2Length+subX0rowcol3Length+subX0rowcol4Length);

% TestX0 = [X011  X012  X013  X014;
%           X012' X022  X023  X024;
%           X013' X023' X033  X034;
%           X014' X024' X034' X044]
%     
% Test = TestX0-X0
% Matriseoppbygging stemmer

%% 1st LMI in (46) in Bao, according to Appendix D eq (70)
BRL = newlmi; %LMI form the Bounded Real Lemma

% A11
% Konstant matrise
A110 = (Ag'*X011'+X011*Ag)-(Cg'*Dk'*Bg'*X011'+X011*Bg*Dk*Cg)-(Cg'*Bp'*X012'+X012*Bp*Cg)+(Cg'*Bo'*X013'+X013*Bo*Cg)-(Cg'*Bk'*X014'+X014*Bk*Cg);
lmiterm([BRL 1 1 0],A110)
lmiterm([BRL 1 1 dX11],1,Ag,'s')
lmiterm([BRL 1 1 dX11],1,-Bg*Dk*Cg,'s')
lmiterm([BRL 1 1 dDk],-X011*Bg,Cg,'s')
lmiterm([BRL 1 1 dX12],1,-Bp*Cg,'s')
lmiterm([BRL 1 1 dX13],1,Bo*Cg,'s')
lmiterm([BRL 1 1 dX14],1,-Bk*Cg,'s')
% Element dobbeltsjekket
% A12
% Konstant matrise
A120 = Ag'*X012-Cg'*Dk'*Bg'*X012-Cg'*Bp'*X022+Cg'*Bo'*X023'-Cg'*Bk'*X024'+X012*Ap;
lmiterm([BRL 1 2 0],A120)
lmiterm([BRL 1 2 dX12],Ag',1)
lmiterm([BRL 1 2 dDk],-Cg',Bg'*X012) % dDk er symmetrisk -> dDk=dDk'
lmiterm([BRL 1 2 dX12],-Cg'*Dk'*Bg',1)
lmiterm([BRL 1 2 dX22],-Cg'*Bp',1)
lmiterm([BRL 1 2 dX32],Cg'*Bo',1)% dX32 = dX23'
lmiterm([BRL 1 2 dX42],-Cg'*Bk',1) % dX42 = dX24'
lmiterm([BRL 1 2 dX12],1,Ap)
% Element dobbeltsjekket
% A13
% Konstant matrise
A130 = Ag'*X013-Cg'*Dk'*Bg'*X013-Cg'*Bp'*X023+Cg'*Bo'*X033-Cg'*Bk'*X034'+X013*Ao;
lmiterm([BRL 1 3 0],A130)
lmiterm([BRL 1 3 dX13],Ag',1)
lmiterm([BRL 1 3 dDk],-Cg',Bg'*X013) % dDk er symmetrisk -> dDk=dDk'
lmiterm([BRL 1 3 dX13],-Cg'*Dk'*Bg',1)
lmiterm([BRL 1 3 dX23],-Cg'*Bp',1)
lmiterm([BRL 1 3 dX33],Cg'*Bo',1)
lmiterm([BRL 1 3 dX43],-Cg'*Bk',1) % dX43 = dX34'
lmiterm([BRL 1 3 dX13],1,Ao)
% Element dobbeltsjekket
% A14
% Konstant matrise
A140 = Ag'*X014-Cg'*Dk'*Bg'*X014-Cg'*Bp'*X024+Cg'*Bo'*X034-Cg'*Bk'*X044+X011*Bg*Ck+X014*Ak;
lmiterm([BRL 1 4 0],A140)
lmiterm([BRL 1 4 dX14],Ag',1)
lmiterm([BRL 1 4 dDk],-Cg',Bg'*X014) % dDk er symmetrisk -> dDk=dDk'
lmiterm([BRL 1 4 dX14],-Cg'*Dk'*Bg',1)
lmiterm([BRL 1 4 dX24],-Cg'*Bp',1)
lmiterm([BRL 1 4 dX34],Cg'*Bo',1)
lmiterm([BRL 1 4 dX44],-Cg'*Bk',1)
lmiterm([BRL 1 4 dX11],1,Bg*Ck)
lmiterm([BRL 1 4 dCk],X011*Bg,1)
lmiterm([BRL 1 4 dAk],X014,1)
lmiterm([BRL 1 4 dX14],1,Ak)
% Element dobbeltsjekket
% A15
% Konstant matrise
A150 = X011*Bg*Dk+X012*Bp+X014*Bk;
lmiterm([BRL 1 5 0],A150)
lmiterm([BRL 1 5 dDk],X011*Bg,1)
lmiterm([BRL 1 5 dX11],1,Bg*Dk)
lmiterm([BRL 1 5 dX12],1,Bp)
lmiterm([BRL 1 5 dX14],1,Bk)
% Element dobbeltsjekket
% A16
lmiterm([BRL 1 6 gamma],-Cg'*Dp',1)
% Element dobbeltsjekket
% A17
lmiterm([BRL 1 7 0],Cg'*Do')
% Element dobbeltsjekket
% A22
% Konstant matrise
A220 = Ap'*X022+X022*Ap;
lmiterm([BRL 2 2 0],A220)
lmiterm([BRL 2 2 dX22],1,Ap,'s')
% Element dobbeltsjekket
% A23
% Konstant matrise
A230 = Ap'*X023+X023*Ao;
lmiterm([BRL 2 3 0],A230)
lmiterm([BRL 2 3 dX23],Ap',1)
lmiterm([BRL 2 3 dX23],1,Ao)
% Element dobbeltsjekket
% A24
% Konstant matrise
A240 = Ap'*X024+X012'*Bg*Ck+X024*Ak;
lmiterm([BRL 2 4 0],A240)
lmiterm([BRL 2 4 dX24],Ap',1)
lmiterm([BRL 2 4 dX21],1,Bg*Ck) % dX21 = dX12'
lmiterm([BRL 2 4 dCk],X012'*Bg,1)
lmiterm([BRL 2 4 dX24],1,Ak)
lmiterm([BRL 2 4 dAk],X024,1)
% Element dobbeltsjekket
% A25
% Konstant matrise
A250 = X012'*Bg*Dk+X022*Bp+X024*Bk;
lmiterm([BRL 2 5 0],A250)
lmiterm([BRL 2 5 dX21],1,Bg*Dk) % dX21 = dX12'
lmiterm([BRL 2 5 dDk],X012'*Bg,1)
lmiterm([BRL 2 5 dX22],1,Bp)
lmiterm([BRL 2 5 dX24],1,Bk)
% Element dobbeltsjekket
% A26
lmiterm([BRL 2 6 gamma],Cp',1)
% Element dobbeltsjekket
% A27
lmiterm([BRL 2 7 0],0) % kan hende jeg m bygge en nullermatrise
% Element dobbeltsjekket
% A33
% Konstant matrise
A330 = Ao'*X033+X033*Ao;
lmiterm([BRL 3 3 0],A330)
lmiterm([BRL 3 3 dX23],1,Ao,'s')
% Element dobbeltsjekket
% A34
% Konstant matrise
A340 = Ao'*X034+X013'*Bg*Ck+X034*Ak;
lmiterm([BRL 3 4 0],A340)
lmiterm([BRL 3 4 dX34],Ao',1) 
lmiterm([BRL 3 4 dX31],1,Bg*Ck) % dX31 = dX13'
lmiterm([BRL 3 4 dCk],X013'*Bg,1)
lmiterm([BRL 3 4 dX34],1,Ak)
lmiterm([BRL 3 4 dAk],X034,1)
% Element dobbeltsjekket
% A35
% Konstant matrise
A350 = X013'*Bg*Dk+X023'*Bp+X034*Bk;
lmiterm([BRL 3 5 0],A350)
lmiterm([BRL 3 5 dX31],1,Bg*Dk) % dX31 = dX13'
lmiterm([BRL 3 5 dDk],X013'*Bg,1)
lmiterm([BRL 3 5 dX32],1,Bp) % dX32 = dX23'
lmiterm([BRL 3 5 dX34],1,Bk)
% Element dobbeltsjekket
% A36
lmiterm([BRL 3 6 0],0) % kan hende jeg m bygge en nullermatrise
% Element dobbeltsjekket
% A37
lmiterm([BRL 3 7 0],Co')
% Element dobbeltsjekket
% A44
% Konstant matrise
A440 = Ck'*Bg'*X014+X014'*Bg*Ck+Ak'*X044+X044*Ak;
lmiterm([BRL 4 4 0],A440)
lmiterm([BRL 4 4 dX41],1,Bg*Ck,'s')% dX41 = dX14'
lmiterm([BRL 4 4 dCk],X014'*Bg,1,'s')
lmiterm([BRL 4 4 dX44],1,Ak,'s')
lmiterm([BRL 4 4 dAk],X044,1,'s')
% Element dobbeltsjekket
% A45
% Konstant matrise
A450 = X014'*Bg*Dk+X024'*Bp+X044*Bk;
lmiterm([BRL 4 5 0],A450)
lmiterm([BRL 4 5 dX41],1,Bg*Dk) % dX41 = dX14'
lmiterm([BRL 4 5 dDk],X014'*Bg,1)
lmiterm([BRL 4 5 dX42],1,Bp) % dX42 = dX24'
lmiterm([BRL 4 5 dX44],1,Bk)
% Element dobbeltsjekket
% A46
lmiterm([BRL 4 6 0],0) % kan hende jeg m bygge en nullermatrise
% Element dobbeltsjekket
% A47
lmiterm([BRL 4 7 0],0) % kan hende jeg m bygge en nullermatrise
% Element dobbeltsjekket
% A55
lmiterm([BRL 5 5 0],-1) % kan hende jeg m bygge en identitetsmatrise
% Element dobbeltsjekket
% A56
lmiterm([BRL 5 6 gamma],Dp',1)
% Element dobbeltsjekket
% A57
lmiterm([BRL 5 7 0],0) % kan hende jeg m bygge en nullermatrise
% Element dobbeltsjekket
% A66
lmiterm([BRL 6 6 0],-1) % kan hende jeg m bygge en identitetsmatrise
% Element dobbeltsjekket
% A67
lmiterm([BRL 6 7 0],0) % kan hende jeg m bygge en nullermatrise
% Element dobbeltsjekket
% A77
lmiterm([BRL 7 7 0],-1) % kan hende jeg m bygge en identitetsmatrise
% Element dobbeltsjekket

%% 2nd LMI in (46), dX+X0 > 0
% LMI dobbeltsjekket
Xpos = newlmi;
lmiterm([-Xpos 1 1 dXtot],1,1);
lmiterm([-Xpos 1 1 0],X0);
 
%% 3nd LMI, gamma has to be positive
% LMI dobbeltsjekket
gammapos = newlmi;
lmiterm([-gammapos 1 1 gamma],1,1);

%% 4nd LMI, bounds on magnitude at dAk,dCk dDk dX, eq(50) Bao
% Positive and negative bound required at the matrix elements
% Posetive bound
for k=1:loopno
    dakpos(k) = newlmi;
    lmiterm([dakpos(k) 1 1 daki(k)],1,1);
    lmiterm([-dakpos(k) 1 1 0],epsilon);
    
    dck1pos(k) = newlmi;
    lmiterm([dck1pos(k) 1 1 dck1i(k)],1,1);
    lmiterm([-dck1pos(k) 1 1 0],epsilon);
    
    dck2pos(k) = newlmi;
    lmiterm([dck2pos(k) 1 1 dck2i(k)],1,1);
    lmiterm([-dck2pos(k) 1 1 0],epsilon);
    
    ddkpos(k) = newlmi;
    lmiterm([ddkpos(k) 1 1 dDki(k)],1,1);
    lmiterm([-ddkpos(k) 1 1 0],epsilon); 
end
% Negative bound
for k=1:loopno
    dakposn(k) = newlmi;
    lmiterm([-dakposn(k) 1 1 daki(k)],1,1);
    lmiterm([dakposn(k) 1 1 0],-epsilon);
    
    dck1posn(k) = newlmi;
    lmiterm([-dck1posn(k) 1 1 dck1i(k)],1,1);
    lmiterm([dck1posn(k) 1 1 0],-epsilon);
    
    dck2posn(k) = newlmi;
    lmiterm([-dck2posn(k) 1 1 dck2i(k)],1,1);
    lmiterm([dck2posn(k) 1 1 0],-epsilon);
    
    ddkposn(k) = newlmi;
    lmiterm([-ddkposn(k) 1 1 dDki(k)],1,1);
    lmiterm([ddkposn(k) 1 1 0],-epsilon); 
end
% Positive bound
for k = 1:length(Acl)
    for i = k:length(Acl)
        dXiios(k,i) = newlmi;
        lmiterm([dXiios(k,i) 1 1 dX(k,i)],1,1);
        lmiterm([-dXiios(k,i) 1 1 0],epsilon); 
    end
end

% Negative bound
for k = 1:length(Acl)
    for i = k:length(Acl)
        dXiiosn(k,i) = newlmi;
        lmiterm([-dXiiosn(k,i) 1 1 dX(k,i)],1,1);
        lmiterm([dXiios(k,i) 1 1 0],-epsilon); 
    end
end
%% Initialiserer LMI systemet
lmisys = getlmis;
%% Henter ut C vektor
ndec = decnbr(lmisys);
c = zeros(ndec,1);

% Identifiserer hvor gamma er i x vektoren
gamma1 = 0;
i =1; 
while gamma1==0
  [gamma1 Vak Vck Vdk VX] = defcx(lmisys,i,gamma,dAk,dCk,dDk,dXtot);  
end
c(i) = -1;
clear i;

%% Finding optimal step
% M hente ut en feasable lsning som startpunkt for mincx. Hvis ikke, blir
% mincx infeaseble.
options=zeros(1,5);  % default value for all parameters 
options(2)=200; % Increasing maximum iterations
options(3)=-1; % Feasibility radius R set to flexible mode
disp('In optStep: Feasibility checks starts')
[tmin,xfeas] = feasp(lmisys,options);

%% Displays and returning results
if tmin >= 1e-7
    disp('ERROR in optStep: Optimization step infeaseble')
    valdAk = 0;
    valdCk = 0;
    valdDk = 0;
    valdX = 0;
    valgammak = -1;
elseif tmin<0
    disp('In optStep: Optimization step feaseble, optimization starts')
    options = zeros(1,5);
    [copt,xopt] = mincx(lmisys,c,options,xfeas);
    
    valdAk = dec2mat(lmisys,xopt,dAk);
    valdCk = dec2mat(lmisys,xopt,dCk);
    valdDk = dec2mat(lmisys,xopt,dDk);
    valdX = dec2mat(lmisys,xopt,dXtot);
    valgammak = dec2mat(lmisys,xopt,gamma);
else
    disp('WARNING in optStep: Optimization step feaseble, optimization starts')
    options = zeros(1,5);
    [copt,xopt] = mincx(lmisys,c,options,xfeas);
    
    valdAk = dec2mat(lmisys,xopt,dAk);
    valdCk = dec2mat(lmisys,xopt,dCk);
    valdDk = dec2mat(lmisys,xopt,dDk);
    valdX = dec2mat(lmisys,xopt,dXtot);
    valgammak = dec2mat(lmisys,xopt,gamma);
end
%% !!NB!! M SJEKKE ||Ak||..osv <= epsilon ligning (50)!!NB!!