%% Initialiserer systemet G(s) og diagonalkontroller K(s). 
% Definerer tilhrende systemmatriser for G(s) og init verdier for 
% systemmatriser for kontrller K(s)
SystemArt; 


%%
% ------------------- %
% | Algoritmesteg 1 | %
% ------------------- %
% Finding start values for gamma and X, Solving (29) Bao
disp('In run: Finding initial values for gamma and X')
[gamma0 X0] = brlOpt1step(Ag,Bg,Cg,Dg,Ap,Bp,Cp,Dp,Ao,Bo,Co,Do,Ak,Bk,Ck,Dk);

% Declaring parameters (step 1 in Bao)
% Convergence tolerance
psi= 0.0001;
% Max iterations
nitMax = 100;
% Iterations counter
k = 0;
% Solution radius
epsilon = 1e-3;
% Solution radius uppdate variables
alpha = 0.95;
beta = 1.05;
%% Main algoritm loop
while 1
    if k>(nitMax+100)
        disp('ERROR in run: Infinite loop')
        break;
    end
    % ------------------- %
    % | Algoritmesteg 2 | %
    % ------------------- %
%     Calculating optimal step
    disp(['In run: AlgStep2, Step number k=',num2str(k)])
    disp('In run: AlgStep2, Finding optimal step value.')
    [valdAk,valdCk,valdDk,valgamma,valdXk]= optStep(X0,Ag,Bg,Cg,Dg,Ao,Bo,Co,Do,Ap,Bp,Cp,Dp,Ak,Bk,Ck,Dk,epsilon);
    if valgamma==-1
        disp(['In run: AlgStep2, Step number k=',num2str(k)])
        disp('ERROR in run: OptStep [Equation (46)] is infeaseble')
        break;
    end
    % ------------------- %
    % | Algoritmesteg 3 | %
    % ------------------- %
    % Updating variables
    disp(['In run: AlgStep3, Step number k=',num2str(k)])
    disp('In run: AlgStep3, Updating control parameters.')
    Ak0 = Ak;
    Ck0 = Ck;
    Dk0 = Dk;
    X00 = X0;
    gamma00 = gamma0;
    
    Ak = Ak+valdAk;
    Ck = Ck+valdCk;
    Dk = Dk+valdDk;
    X0 = X0+valdXk;
    disp(['In run: AlgStep3, Step number k=',num2str(k)])
    disp('In run: AlgStep3, Testing feasebility to new values.')
    % Testing feasibility for problem wiht given values for Ak Bk Ck Dk og X0(29)
    [tmin xfeas] = brlfeas(Ag,Bg,Cg,Dg,Ap,Bp,Cp,Dp,Ao,Bo,Co,Do,Ak,Bk,Ck,Dk,X0);
   
    % If solution is feasable
    if tmin<0
        if abs(gamma0-valgamma)<= psi
           disp(['In run: AlgStep3, Step number k=',num2str(k)])
           disp('EXCITEMENT in run: abs(gamma0-valgamma)<= psi: Acceptable solution obtained! :-)')
           % calculate the PID parameters from Ak Bk Ck Dk
           [kp tauI tauD tauL] = PIDMatrix2Param(Ak,Ck,Dk);
           break;
        elseif abs(gamma0-valgamma)> psi
           disp('In run: AlgStep3, abs(gamma0-valgamma)> psi: epsilon = alpha*epsilon and calculating optimal step.')
           k = k+1;
           % Update feasibility radii
           epsilon = alpha*epsilon;
           if k>nitMax
              disp('WARNING in run: k>nitMax. Increase max iterations or psi')
              break;
           end
           % Go to step 2
        else
            disp('ERROR in run: unexpected value of abs(gamma0-valgamma. Change code')
            break;
        end
    % If solution is infeaseble
    elseif tmin>=0
        disp(['WARNING: tmin = ',num2str(tmin),', -> tmin > 0: not strictly feasible, change variable radii'])
        if k>nitMax
            disp(['WARNING: tmin = ',num2str(tmin),', -> tmin > 1e-7: not feasible, and ',num2str(k),' >nitMax']) 
            disp('-> Infeaseble solutions in the neighborhood. Choose a different startpoint for Ak Ck Dk.')
            break;
        else
        % elseif k<=nitMax
            disp(['In run: AlgStep3, Step number k=',num2str(k)])
            disp('-> Infeaseble solution with epsilon. Increasing epsilon, and calculate step with previous values of Ak Ck Dk and X.')
            k = k+1;
            % Update feasibility radii
            epsilon = beta*epsilon;
            % Rejecting solutions, set optimation variables to preveous
            % values
            Ak = Ak0;
            Ck = Ck0;
            Dk = Dk0;
            X0 = X00;
            gamma0 = gamma00;
            % Go to step2
        end 
    % If something unexpected should happend
    else
        disp('ERROR: unexpected value of tmin. Change code')  
        break;
    end
end

